/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#ifndef __SETTINGS_H__
#define __SETTINGS_H__

#include <qobject.h>
#include <qcolor.h>
#include <qfont.h>
#include <qfile.h>

#include "datatypes.h"
#include "skin.h"
#include "adder.h"

class ConfigFile
{
	public:
		ConfigFile(const QString& path)
		{
			this->path = path;
		}
		
		void read()
		{
			QFile f(path);
			if (!f.open(IO_ReadOnly))
				return;
			
			QTextStream ts(&f);
			ts.setEncoding(QTextStream::UnicodeUTF8);
			
			while (!ts.atEnd()) {
				QString s = ts.readLine();
				if (s.length() == 0 || s.left(1) == "#")
					continue;
				
				QString name = s.left(s.find("="));
				_variables[name] = s.mid(name.length() + 1, s.find(";") - name.length() - 1);
			}
			
			f.close();
		}
		
		void write()
		{
			QFile f(path);
			f.open(IO_WriteOnly);
			QTextStream ts(&f);
			ts.setEncoding(QTextStream::UnicodeUTF8);
			
			QStringList keys = _variables.keys();
			for (unsigned int i = 0; i < keys.count(); i++)
				ts << keys[i] << "=" << _variables[keys[i]] << ";\n";
			
			f.close();
		}
		
		const bool contains(const QString& key) const
		{return _variables.contains(key);}
		
		QString get(const QString& key, QString def) const
		{if (!contains(key)) return def; return _variables[key];}
		
		int get(const QString& key, int def) const
		{if (!contains(key)) return def; return _variables[key].toInt();}
		
		bool get(const QString& key, bool def) const
		{if (!contains(key)) return def; return (_variables[key].toInt() != 0);}
		
		void set(const QString& key, QString value)
		{_variables[key] = value;}
		
		void set(const QString& key, int value)
		{_variables[key] = QString::number(value);}
		
		void set(const QString& key, bool value)
		{_variables[key] = (value ? "1" : "0");}
	
	private:
		QString path;
		QMap<QString, QString> _variables;
};

class Settings
	: public QObject
{
	Q_OBJECT
	
	signals:
		void autoColorsChange(bool);
		void bgChange(QColor);
		void fgChange(QColor);
		void fontChange(QFont);
		void autoFontChange(bool);
		void fontFaceChange(const QString&);
		void fontSizeChange(int);
		void skinChange(const QString&);
		void cacheTimeoutChange(int);
		void headlineTimeoutChange(int);
		void scrollTimeoutChange(int);
		void sitesChange(SiteList);
		void monitoringChange(bool);
		void scrollingChange(bool);
		void tooltipsChange(bool);
		void underliningChange(bool);
		void browserChange(const QString&);
	
	public slots:
		void load();
		void save();
		void setAutoColors(bool);
		void setBgColor(QColor);
		void setFgColor(QColor);
		void setAutoFont(bool);
		void setFontFace(const QString&);
		void setFontSize(int);
		void setSkin(const QString&);
		void setCacheTimeout(int);
		void setHeadlineTimeout(int);
		void setScrollTimeout(int);
		void appendSite(Site);
		void indexSite(Site, int);
		void removeSite(Site);
		void disableSite(QString);
		void setTitle(QString, QString);
		void enableMonitoring(bool);
		void enableUnderlining(bool);
		void enableTooltips(bool);
		void enableScrolling(bool);
		void modifyUrl(QString, QString);
		void setBrowser(const QString&);
		void setStartPos(const QPoint&);
	
	public:
		Settings(QObject* parent = 0, const char* name = 0);
		void opmlimport(QString);
		void opmlexport(QString);
		bool autoColors() const;
		QColor bg() const;
		QColor fg() const;
		bool autoFont() const;
		QString fontFace() const;
		int fontSize() const;
		QString skin() const;
		int cacheTimeout() const;
		int headlineTimeout() const;
		int scrollTimeout() const;
		SiteList sites() const;
		bool monitoring() const;
		bool underlining() const;
		bool tooltips() const;
		bool scrolling() const;
		QString browser() const;
		QPoint startPos() const;
	
	private:
		void emitFontChange();
		void emitColorChange();
		QColor _bg;
		QColor _fg;
		bool _autocolors;
		QString _fontface;
		int _fontsize;
		bool _autofont;
		QString _skin;
		int _cachetimeout;
		int _headlinetimeout;
		int _scrolltimeout;
		SiteList _sites;
		bool _monitoring;
		bool _underlining;
		bool _tooltips;
		bool _scrolling;
		QString _browser;
		QPoint _startpos;
};

#endif
